package br.ufsc.uid2uml;

import br.ufsc.uid2uml.UMLDiagram.*;
//import br.ufsc.uid.elementsUID.UID;
//import br.ufsc.uid.elementsUID.UIDException;

import java.io.File;
import java.util.List;
//import java.io.FileOutputStream;
//import java.io.OutputStreamWriter;
//import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.OutputKeys;
//import javax.xml.stream.XMLOutputFactory;
//import javax.xml.stream.XMLStreamWriter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
//import org.w3c.dom.NamedNodeMap;
//import org.xml.sax.InputSource;

import java.util.Map;
import java.util.HashMap;


public class ClassDiagramToXMIConverter {
	
	private static ClassDiagramToXMIConverter converter;
	
	private ClassDiagramToXMIConverter() {
	}	
	
	public static ClassDiagramToXMIConverter getConverter(){
		if (converter == null){
			converter = new ClassDiagramToXMIConverter();
		}
		return converter;
	}

	public void convertClassDiagramToXMI (ClassDiagram diagram) throws Exception {
		try{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		DocumentBuilder docBuilder = factory.newDocumentBuilder();
		
		Document d = docBuilder.newDocument();
		
// <XMI xmi.version="1.2" xmlns:UML="org.omg/UML/1.4">
		Element xmiElement = d.createElement("XMI");
		xmiElement.setAttribute("xmi.version", "1.2");
		xmiElement.setAttribute("xmlns:UML", "org.omg.xmi.namespace.UML");
		//xmiElement.setAttribute("xmlns:UML", "org.omg/UML/1.4");
		d.appendChild(xmiElement);
		
		this.createHeaderElements (d, xmiElement);
		
		this.createContentElements (d, xmiElement, diagram);

		// write the content into xml file
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		transformer.setOutputProperty(OutputKeys. INDENT, "yes");

		
		DOMSource source = new DOMSource(d);
		String fileName = diagram.getName() + ".xmi";
		StreamResult result = new StreamResult(new File(fileName));
 
		// Output to console for testing
		// StreamResult result = new StreamResult(System.out);
 
		transformer.transform(source, result);
 
		System.out.println("Escreveu o arquivo xml!");
 
	  } catch (ParserConfigurationException pce) {
		pce.printStackTrace();
	  } catch (TransformerException tfe) {
		tfe.printStackTrace();
	  }
	}

		
	private void createHeaderElements (Document d, Element xmiElement){
// <XMI.header>
		Element headerElement = d.createElement("XMI.header");
		xmiElement.appendChild(headerElement);
// <XMI.documentation>
		Element documentationElement = d.createElement("XMI.documentation");
		headerElement.appendChild(documentationElement);
// <XMI.exporter>acho que aqui se coloca da organizao que gera o xmi</XMI.exporter>
		Element exporterElement = d.createElement("XMI.exporter");
		exporterElement.appendChild(d.createTextNode(""));
		documentationElement.appendChild(exporterElement);
// <XMI.metamodel xmi.name="UML" xmi.version="1.4"/>
		Element metamodelElement = d.createElement("XMI.metamodel");
		metamodelElement.setAttribute("xmi.name", "UML");
		metamodelElement.setAttribute("xmi.version", "1.4");
		headerElement.appendChild(metamodelElement);
	}
	
	private void createContentElements (Document d, Element xmiElement, ClassDiagram diagram){
		
//		 <XMI.content>
		Element contentElement = d.createElement("XMI.content");
		xmiElement.appendChild(contentElement);
//		  <UML:Model xmi.id="M.1" name="address" visibility="public"
//		              isSpecification="false" isRoot="false"
//		              isLeaf="false" isAbstract="false">
		
		Element modelElement = d.createElement("UML:Model");
		modelElement.setAttribute("xmi.id", "Model.1");
		modelElement.setAttribute("name", diagram.getName());
//		modelElement.setAttribute("name", "untitledModel");
		//precisa do atributo visibility?
		modelElement.setAttribute("visibility", "public");
		modelElement.setAttribute("isSpecification", "false");
		modelElement.setAttribute("isRoot", "false");
		modelElement.setAttribute("isLeaf", "false");
		modelElement.setAttribute("isAbstract", "false");
		contentElement.appendChild(modelElement);

//		   <UML:Namespace.ownedElement>
		Element namespaceElement = d.createElement("UML:Namespace.ownedElement");
		modelElement.appendChild(namespaceElement);
		
		//For each class of the class diagram, convert to xmi
		
		int classNumber = 0;
		int attributeNumber = 0;
		int associationNumber = 0;
		int associationEndNumber = 0;
		int multiplicityNumber = 0;
		int rangeNumber = 0;
		
		Map<String,String> mapperClassNameToId = new HashMap<String, String>();
		
		List<br.ufsc.uid2uml.UMLDiagram.Class> classes = diagram.getClasses();
		for (int i=0, sizeC=classes.size(); i<sizeC; i++){
			br.ufsc.uid2uml.UMLDiagram.Class cl = classes.get(i);

//		    <UML:Class xmi.id="C.1" name="address" visibility="public"
//            isSpecification="false" namespace="M.1" isRoot="true"
//            isLeaf="true" isAbstract="false" isActive="false">
			
			Element classElement = d.createElement("UML:Class");
			namespaceElement.appendChild(classElement);
			classNumber++;
			String idClass = "Class."+classNumber;
			mapperClassNameToId.put(cl.getName(), idClass);
			classElement.setAttribute("xmi.id", idClass);
			classElement.setAttribute("name", cl.getName());
			classElement.setAttribute("visibility", "public");
			classElement.setAttribute("isSpecification", "false");
			classElement.setAttribute("isRoot", "false");
		    classElement.setAttribute("isLeaf", "false");
			classElement.setAttribute("isAbstract", "false");
			classElement.setAttribute("isActive", "false");

//	     <UML:Classifier.feature>
		    Element classifierElement = d.createElement("UML:Classifier.feature");
		    classElement.appendChild(classifierElement);	
			
		//For each attribute of the class, convert to xmi

			List<Attribute> attributes = cl.getAttributes();
			for (int j=0, sizeA=attributes.size(); j<sizeA; j++){
				Attribute at = attributes.get(j);
				
//			      <UML:Attribute xmi.id="A.1" name="name" visibility="private"
//                isSpecification="false" ownerScope="instance"/>

				Element attributeElement = d.createElement("UML:Attribute");
				attributeNumber++;
				attributeElement.setAttribute("xmi.id", "Attr."+classNumber+"."+attributeNumber);
				attributeElement.setAttribute("name", at.getName());
				attributeElement.setAttribute("visibility", "private");
				attributeElement.setAttribute("isSpecification", "false");
				attributeElement.setAttribute("ownerScope", "instance");
				attributeElement.setAttribute("changeability", "changeable");
				attributeElement.setAttribute("targetScope", "instance");
				classifierElement.appendChild(attributeElement);

				//parte da multiplicidade do atributo
				Element structuralFeatureElement = d.createElement("UML:StructuralFeature.multiplicity");
				attributeElement.appendChild(structuralFeatureElement);
				Element multiplicityElement = d.createElement("UML:Multiplicity");
				multiplicityElement.setAttribute("xmi.id", "M."+classNumber+"."+attributeNumber);
				structuralFeatureElement.appendChild(multiplicityElement);
				Element rangeElement = d.createElement("UML:Multiplicity.range");
				multiplicityElement.appendChild(rangeElement);
				Element multiplicityRangeElement = d.createElement("UML:MultiplicityRange");
				multiplicityRangeElement.setAttribute("xmi.id", "R."+classNumber+"."+attributeNumber);
				multiplicityRangeElement.setAttribute("lower", "1");
				multiplicityRangeElement.setAttribute("upper", "1");
				rangeElement.appendChild(multiplicityRangeElement);
			}
		}	
			
		//For each association of the diagram, convert to xmi

		List<Association> associations = diagram.getAssociations();
		for (int k=0, sizeAS=associations.size(); k<sizeAS; k++){
			Association as = associations.get(k);

//	        <UML:Association xmi.id = '-64--88-1-36-67e0f5a1:133eefe9745:-8000:0000000000001BE6'
//                name = '' isSpecification = 'false' isRoot = 'false' isLeaf = 'false' isAbstract = 'false'>
			Element associationElement = d.createElement("UML:Association");
			associationNumber++;
			associationElement.setAttribute("xmi.id", "Ass."+associationNumber);
			associationElement.setAttribute("name", as.getName());
			associationElement.setAttribute("isSpecification", "false");
			associationElement.setAttribute("isRoot", "false");
			associationElement.setAttribute("isLeaf", "false");
			associationElement.setAttribute("isAbstract", "false");
			namespaceElement.appendChild(associationElement);

//            <UML:Association.connection>
			Element connectionElement = d.createElement("UML:Association.connection");
			associationElement.appendChild(connectionElement);

			//PRIMEIRO LADO DA ASSOCIAO
			
//            <UML:AssociationEnd xmi.id = '-64--88-1-36-67e0f5a1:133eefe9745:-8000:0000000000001BE7'
//                  visibility = 'public' isSpecification = 'false' isNavigable = 'true' ordering = 'unordered'
//                  aggregation = 'none' targetScope = 'instance' changeability = 'changeable'>
			Element associationEndElement1 = d.createElement("UML:AssociationEnd");
			associationEndNumber++;
			associationEndElement1.setAttribute("xmi.id", "AssEnd."+associationNumber+"."+associationEndNumber);
			associationEndElement1.setAttribute("visibility", "public");
			associationEndElement1.setAttribute("isSpecification", "false");
			associationEndElement1.setAttribute("isNavigable", "true");
			associationEndElement1.setAttribute("ordering", "unordered");
			associationEndElement1.setAttribute("aggregation", "none");
			associationEndElement1.setAttribute("targetScope", "instance");
			associationEndElement1.setAttribute("changeability", "changeable");
			connectionElement.appendChild(associationEndElement1);

//            <UML:AssociationEnd.multiplicity>
//            <UML:Multiplicity xmi.id = '-64--88-1-36-67e0f5a1:133eefe9745:-8000:0000000000001BE8'>
//              <UML:Multiplicity.range>
//                <UML:MultiplicityRange xmi.id = '-64--88-1-36-67e0f5a1:133eefe9745:-8000:0000000000001BE9'
//                  lower = '1' upper = '1'/>
//              </UML:Multiplicity.range>
//            </UML:Multiplicity>
//          </UML:AssociationEnd.multiplicity>
			Element associationEndMultiplicityElement1 = d.createElement("UML:AssociationEnd.multiplicity");
			associationEndElement1.appendChild(associationEndMultiplicityElement1);
			Element multiplicityElement1 = d.createElement("UML:Multiplicity");
			multiplicityNumber++;
			multiplicityElement1.setAttribute("xmi.id", "Mult."+multiplicityNumber);
			associationEndMultiplicityElement1.appendChild(multiplicityElement1);

			Element multiplicityRangeElement1 = d.createElement("UML:Multiplicity.range");
			multiplicityElement1.appendChild(multiplicityRangeElement1);

			Element rangeElement1 = d.createElement("UML:MultiplicityRange");
			rangeNumber++;
			rangeElement1.setAttribute("xmi.id", "Range."+rangeNumber);
// MODIFIQUEI AQUI...
//			rangeElement1.setAttribute("lower", "1");
//			rangeElement1.setAttribute("upper", "1");
			rangeElement1.setAttribute("lower", ""+as.getSourceMinCardinality());
			rangeElement1.setAttribute("upper", ""+as.getSourceMaxCardinality());
			multiplicityRangeElement1.appendChild(rangeElement1);

//          <UML:AssociationEnd.participant>
//            <UML:Class xmi.idref = '-64--88-1-36-67e0f5a1:133eefe9745:-8000:0000000000001BE4'/>
//          </UML:AssociationEnd.participant>

			Element participantElement1 = d.createElement("UML:AssociationEnd.participant");
			associationEndElement1.appendChild(participantElement1);
			Element participantClassElement1 = d.createElement("UML:Class");
			String idClass = mapperClassNameToId.get(as.getSourceClass().getName());
			participantClassElement1.setAttribute("xmi.idref", idClass);
			participantElement1.appendChild(participantClassElement1);
			
			//SEGUNDO LADO DA ASSOCIAO
			
			Element associationEndElement2 = d.createElement("UML:AssociationEnd");
			associationEndNumber++;
			associationEndElement2.setAttribute("xmi.id", "AssEnd."+associationNumber+"."+associationEndNumber);
			associationEndElement2.setAttribute("visibility", "public");
			associationEndElement2.setAttribute("isSpecification", "false");
			associationEndElement2.setAttribute("isNavigable", "true");
			associationEndElement2.setAttribute("ordering", "unordered");
			associationEndElement2.setAttribute("aggregation", "none");
			associationEndElement2.setAttribute("targetScope", "instance");
			associationEndElement2.setAttribute("changeability", "changeable");
			connectionElement.appendChild(associationEndElement2);

			Element associationEndMultiplicityElement2 = d.createElement("UML:AssociationEnd.multiplicity");
			associationEndElement2.appendChild(associationEndMultiplicityElement2);
			Element multiplicityElement2 = d.createElement("UML:Multiplicity");
			multiplicityNumber++;
			multiplicityElement2.setAttribute("xmi.id", "Mult."+multiplicityNumber);
			associationEndMultiplicityElement2.appendChild(multiplicityElement2);

			Element multiplicityRangeElement2 = d.createElement("UML:Multiplicity.range");
			multiplicityElement2.appendChild(multiplicityRangeElement2);

			Element rangeElement2 = d.createElement("UML:MultiplicityRange");
			rangeNumber++;
			rangeElement2.setAttribute("xmi.id", "Range."+rangeNumber);
			// MODIFIQUEI AQUI...
//			rangeElement2.setAttribute("lower", "1");
//			rangeElement2.setAttribute("upper", "1");
			rangeElement2.setAttribute("lower", ""+as.getTargetMinCardinality());
			rangeElement2.setAttribute("upper", ""+as.getTargetMaxCardinality());
			multiplicityRangeElement2.appendChild(rangeElement2);

			Element participantElement2 = d.createElement("UML:AssociationEnd.participant");
			associationEndElement2.appendChild(participantElement2);
			Element participantClassElement2 = d.createElement("UML:Class");
			String idClass2 = mapperClassNameToId.get(as.getTargetClass().getName());
			participantClassElement2.setAttribute("xmi.idref", idClass2);
			participantElement2.appendChild(participantClassElement2);
			
		}	
			
			
	}
		
		
		
	}	
		

	
