package br.ufsc.uid2uml;

/**
 * @author PatriciaVilain
 *
 */
//import java.io.File;
import java.io.FileInputStream;

import br.ufsc.uid2uml.UIDElements.UID;
import br.ufsc.uid2uml.UIDElements.UIDException;


public class Controller {
	
	/**
	 * This method reads the UIDs from the input files and calls the ontology generator
	 * to map the UIDs to the class diagram. 
	 * The resulting ontology (class diagram) is stored in the file arg[0]. 
	 * @param args
	 * 	args[0] is the the name of the class diagram.
	 * 	args[1] to args[n] are files containing UIDs in xml format. All information of 
	 *  these files will be mapped to the same class diagram.
	 * @throws Exception
	 */
	public void execute (String... args) throws Exception{
		if (args == null || args.length < 2) {
			throw new IllegalArgumentException ("no arguments");
		}
		ClassDiagramGenerator classDiagramGenerator = new ClassDiagramGenerator(args[0]);
		UIDXmlReader xmlReader = new UIDXmlReader();
		FileInputStream in; 
		for (int i = 1; i < args.length; i++){
			in = new FileInputStream(args[i]);
			UID uid = xmlReader.read(in);
			classDiagramGenerator.insertUID(uid);
		}	
		classDiagramGenerator.generateClassDiagram();
		classDiagramGenerator.saveIntoFile();
		
	}
	
	public static void main(String[] args) throws Exception {
		Controller manager = new Controller();
		try{
			manager.execute(args);
		}
		catch (IllegalArgumentException e){
			System.out.println("usage: java Controller output_file.xml input_file.xml [input_file.xml*]");
			System.exit(-1);			
		}
		catch (UIDException e){
			System.out.println("UID problems: " + e.getMessage());
			System.exit(-1);						
		}

	}
}
