package br.ufsc.uid2uml;
/**
 * @author Patricia Vilain
 */
import java.io.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;

import br.ufsc.uid2uml.UIDElements.InputDataItem;
import br.ufsc.uid2uml.UIDElements.InputEnumeratedData;
import br.ufsc.uid2uml.UIDElements.InputORDataEntry;
import br.ufsc.uid2uml.UIDElements.InputOptionalDataItem;
import br.ufsc.uid2uml.UIDElements.InputStructure;
import br.ufsc.uid2uml.UIDElements.InputXORDataEntry;
import br.ufsc.uid2uml.UIDElements.OutputDataItem;
import br.ufsc.uid2uml.UIDElements.OutputStructure;
import br.ufsc.uid2uml.UIDElements.UID;
import br.ufsc.uid2uml.UIDElements.UIDException;


import java.util.*;

public class UIDXmlReader {
	
	public UID read (InputStream in){

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(true);
		Document d;
		try {
			DocumentBuilder docbuilder = factory.newDocumentBuilder();
			InputSource is = new InputSource(in);
			d = docbuilder.parse(is);
		} catch (Exception e) {
			e.printStackTrace();
			throw new UIDException("cannot read UID");
		} finally {
			try { if (in != null) in.close(); } catch (Exception ignore) {}
		}
		
		UID uid = null;
		try {	
			Element setUid = d.getDocumentElement(); 		//NOVO
			List uids = _getChildElements(setUid, "UID");	//NOVO
				
			for (Object e : uids) {							//NOVO
				Element uidElement = (Element) e;			//NOVO
				
				//UID
				NamedNodeMap attributes = uidElement.getAttributes();
				String id = _getRequiredAttribute(attributes, "id");
				String name = _getRequiredAttribute(attributes, "name");
				uid = new UID (id, name);
				
				//PRECONDITION
				List preconditionElements = _getChildElements(uidElement,"pre-condition");
				for (int i=0; i<preconditionElements.size(); i++){
					Element preconditionElement = (Element)preconditionElements.get(i);
					String condition = _getRequiredAttribute(preconditionElement,"condition");
					uid.insertPrecondition(condition);
				}
				//POSCONDITION
				List posconditionElements = _getChildElements(uidElement,"pos-condition");
				for (int i=0; i<posconditionElements.size(); i++){
					Element posconditionElement = (Element)posconditionElements.get(i);
					String condition = _getRequiredAttribute(posconditionElement,"condition");
					uid.insertPoscondition(condition);
				}
				
				//SOURCE-UID
				List sourceUIDElements = _getChildElements(uidElement,"source-UID");
//				if (sourceUIDElements.size()!= 0){
				for (int i=0; i<sourceUIDElements.size(); i++){
					String sourceUIDname = _getRequiredAttribute((Element)sourceUIDElements.get(0),"name");
					uid.insertSourceUID(sourceUIDname);
				}
				
				//UID-CALL
				List uidCallElements = _getChildElements(uidElement,"UID-call");
				for (int i=0; i<uidCallElements.size(); i++){
					Element uidCallElement = (Element)uidCallElements.get(i);
					String nameUIDCall = _getRequiredAttribute(uidCallElement,"name");
					String idUIDCall = _getRequiredAttribute(uidCallElement,"id");
					uid.insertUIDCall(idUIDCall, nameUIDCall);
				}

				//TEXTUAL-NOTE
				List textualNoteElements = _getChildElements(uidElement,"textual-note");
				for (int i=0; i<textualNoteElements.size(); i++){
					Element textualNoteElement = (Element)textualNoteElements.get(i);
					String note = _getRequiredAttribute(textualNoteElement,"note");
					uid.insertTextualNote(note);
				}
				
				//CREATING ALL STATES ... to allow the insertion of transitions...
				List initialStateElements = _getChildElements(uidElement,"initial-state"); //just 1 element
				Element initialState = (Element)initialStateElements.get(0);
				String idInitialState = _getRequiredAttribute(initialState,"id");
				uid.insertInitialState(idInitialState);
				
				List stateElements = _getChildElements(uidElement,"state"); 
				for (int i=0; i<stateElements.size(); i++){
					Element state = (Element)stateElements.get(i);
					String idState = _getRequiredAttribute(state,"id");
					uid.insertState(idState);
				}
				
				//LOADING INITIAL-STATE
				initialStateElements = _getChildElements(uidElement,"initial-state"); //just 1 element
				initialState = (Element)initialStateElements.get(0);
				idInitialState = _getRequiredAttribute(initialState,"id");
				loadState (uid, idInitialState, initialState);

				//LOADING STATES
				stateElements = _getChildElements(uidElement,"state"); 
				for (int i=0; i<stateElements.size(); i++){
					Element state = (Element)stateElements.get(i);
					String idState = _getRequiredAttribute(state,"id");
					loadState(uid, idState, state);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new UIDException("cannot read UID");
		} finally {
			try { if (in!= null) in.close(); } catch (Exception ignore) {}
		}
		return uid;
	}
			
	private static void loadState (UID uid, String idState, Element state){
		Node n = state.getFirstChild();
		while (n != null){
			if (n.getNodeType()==Node.ELEMENT_NODE){
				Element elem = (Element)n;
				if (elem.getTagName().equals("data-entry")){
					loadDataEntry (uid, idState, elem);
				}
				if (elem.getTagName().equals("data-output")){
					loadDataOutput (uid, idState, elem);
				}
				if (elem.getTagName().equals("text")){
					loadText (uid, idState, elem);
				}
				//VELHO if (elem.getTagName().equals("optional-text")){
				//VELHO	loadOptionalText (uid, idState, elem);
				//VELHO}
				if (elem.getTagName().equals("transition")){
					loadTransition (uid, idState, elem);
				}
				if (elem.getTagName().equals("substate")) {					//NOVO
					String idSubstate = _getRequiredAttribute(elem, "id");	//NOVO
					uid.insertSubstate(idState, idSubstate);				//NOVO
					loadSubstate(uid, idState, idSubstate, elem);			//NOVO
				}															//NOVO
			}
			n = n.getNextSibling();
		}
	}

	private static void loadSubstate(UID uid, String idState, String idSubstate, Element subState) {
		Node n = subState.getFirstChild();
		while (n != null) {
			if (n.getNodeType() == Node.ELEMENT_NODE) {
				Element elem = (Element) n;
				if (elem.getTagName().equals("data-entry")) {
					loadDataEntryInSubstate(uid, idState, idSubstate, elem);
				}
				if (elem.getTagName().equals("data-output")) {
					loadDataOutputInSubstate(uid, idState, idSubstate, elem);
				}
				if (elem.getTagName().equals("text")) {
					loadTextInSubstate(uid, idState, idSubstate, elem);
				}
				if (elem.getTagName().equals("transition")) {
					loadTransitionInSubstate(uid, idState, idSubstate, elem);
				}
			}
			n = n.getNextSibling();
		}
	}
	
	private static void loadTransitionInSubstate(UID uid, String idState,
			String idSubstate, Element elem) {

		String idTransition = _getRequiredAttribute(elem, "id");
		String target = _getRequiredAttribute(elem, "target");
		String targetType = _getImpliedAttribute(elem, "target-type", "state");
		if (targetType.equals("state"))
			uid.insertTransitionFromSubstateToState(idState, idSubstate,target, idTransition);
		else
			uid.insertTransitionFromSubstateToAnotherUID(idState, idSubstate,target, idTransition);
		String option = _getImpliedAttribute(elem, "option", null);
		if (option != "")
			uid.insertOptionInTransition(idTransition, option);
		String condition = _getImpliedAttribute(elem, "condition", null);
		if (condition != "")
			uid.insertConditionInTransition(idTransition, condition);
		String noReturn = _getImpliedAttribute(elem, "no-return", "false");
		if (noReturn.equals("true"))
			uid.setNoReturnToTransition(idTransition);
	}

	
	private static void loadDataOutputInSubstate(UID uid, String idState,
			String idSubstate, Element dataOutput) {
		Node n = dataOutput.getFirstChild();
		while (n == null || n.getNodeType() != Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if (n != null && n.getNodeType() == Node.ELEMENT_NODE) {
			Element elem = (Element) n;
			if (elem.getTagName().equals("output-data-item")) {
				OutputDataItem dataItem = loadOutputDataItem(elem);
				uid.insertOutputDataItemInSubstate(idState, idSubstate, dataItem);
				// Loading the transitions from the data items
				List outputDataItemTransitionElements = _getChildElements(elem,"transition");
				for (int i = 0; i < outputDataItemTransitionElements.size(); i++) {
					Element dataItemTransitionElement = (Element) outputDataItemTransitionElements.get(i);
					String idTransition = _getRequiredAttribute(dataItemTransitionElement, "id");
					String target = _getRequiredAttribute(dataItemTransitionElement, "target");
					String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputDataItemInSubstateToState(idState, idSubstate,
								dataItem.getName(), target, idTransition);
					else
						uid.insertTransitionFromOutputDataItemInSubstateToAnotherUID(idState, idSubstate, 
								dataItem.getName(), target, idTransition);
					String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition, option);
					String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition, condition);
					String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
				}
			}

			if (elem.getTagName().equals("output-data-item-set")) {
				List dataItemSetElements = _getChildElements(elem,"output-data-item");
				Element elem1 = (Element) dataItemSetElements.get(0);
				OutputDataItem dataItem = loadOutputDataItem(elem1);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				dataItem.definesAsSet();
				dataItem.setCardinalityMin(min);
				dataItem.setCardinalityMax(max);
				uid.insertOutputDataItemInSubstate(idState, idSubstate,dataItem);
				// Loading the transitions from the set of data items
				List outputDataItemSetTransitionElements = _getChildElements(elem, "transition-from-set");
				for (int i = 0; i < outputDataItemSetTransitionElements.size(); i++) {
					Element dataItemSetTransitionElement = (Element) outputDataItemSetTransitionElements.get(i);
					String number = _getRequiredAttribute(dataItemSetTransitionElement, "number");
					List dataItemSetTransitionElements = _getChildElements(dataItemSetTransitionElement, "transition");
					Element dataItemTransitionElement = (Element) dataItemSetTransitionElements.get(0);
					String idTransition = _getRequiredAttribute(
							dataItemTransitionElement, "id");
					String target = _getRequiredAttribute(
							dataItemTransitionElement, "target");
					String targetType = _getImpliedAttribute(
							dataItemTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputDataItemInSubstateToState(idState, idSubstate,
										dataItem.getName(), target,idTransition);
					else
						uid.insertTransitionFromOutputDataItemInSubstateToAnotherUID(idState, idSubstate,
										dataItem.getName(), target,idTransition);
					String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition, option);
					String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
					uid.setNumberOfSelectedElements(idTransition, Integer.parseInt(number));
				}
			}
			if (elem.getTagName().equals("output-structure")) {
				String nameStructure = _getRequiredAttribute(elem, "name");
				String optional = _getRequiredAttribute(elem, "optional");
				OutputStructure structure = new OutputStructure(nameStructure);
				if (optional.equals("true"))
					structure.setOptional(true);
				uid.insertOutputStructureInSubstate(idState, idSubstate,structure);
				List outputStructureElements = _getChildElements(elem,"output-structure-elements");
				Element outputStructureElement = (Element) outputStructureElements.get(0);
				loadOutputStructureInSubstate(uid, idState, idSubstate,nameStructure, outputStructureElement);
				// Loading the transitions from the structure
				List outputStructureTransitionElements = _getChildElements(
						elem, "transition");
				for (int i = 0; i < outputStructureTransitionElements.size(); i++) {
					Element structureTransitionElement = (Element) outputStructureTransitionElements.get(i);
					String idTransition = _getRequiredAttribute(structureTransitionElement, "id");
					String target = _getRequiredAttribute(structureTransitionElement, "target");
					String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputStructureInSubstateToState(idState, idSubstate, nameStructure,
										target, idTransition);
					else
						uid.insertTransitionFromOutputStructureInSubstateToAnotherUID(target, idTransition,idState, idSubstate, nameStructure);
					String option = _getImpliedAttribute(structureTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition, option);
					String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
				}
			}

			if (elem.getTagName().equals("output-structure-set")) {
				String nameStructureSet = _getRequiredAttribute(elem, "name");
				String optional = _getRequiredAttribute(elem, "optional");
				OutputStructure structure = new OutputStructure(nameStructureSet);
				if (optional.equals("true"))
					structure.setOptional(true);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				structure.definesAsSet();
				structure.setCardinalityMin(min);
				structure.setCardinalityMax(max);
				uid.insertOutputStructureInSubstate(idState, idSubstate,structure);
				List outputStructureSetElements = _getChildElements(elem,"output-structure-elements");
				Element outputStructureSetElement = (Element) outputStructureSetElements.get(0);
				loadOutputStructureInSubstate(uid, idState, idSubstate,nameStructureSet, outputStructureSetElement);
				// Loading the transitions from the set of structures
				List outputStructureSetTransitionElements = _getChildElements(elem, "transition-from-set");
				for (int i = 0; i < outputStructureSetTransitionElements.size(); i++) {
					Element structureSetTransitionElement = (Element) outputStructureSetTransitionElements.get(i);
					String number = _getRequiredAttribute(structureSetTransitionElement, "number");
					List structureSetTransitionElements = _getChildElements(structureSetTransitionElement, "transition");
					Element structureTransitionElement = (Element) structureSetTransitionElements.get(0);
					String idTransition = _getRequiredAttribute(structureTransitionElement, "id");
					String target = _getRequiredAttribute(structureTransitionElement, "target");
					String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputStructureInSubstateToState(
							idState, idSubstate, nameStructureSet,target, idTransition);
					else
						uid.insertTransitionFromOutputStructureInSubstateToAnotherUID(
							idState, idSubstate, nameStructureSet,target, idTransition);

					String option = _getImpliedAttribute(structureTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition, option);
					String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
					uid.setNumberOfSelectedElements(idTransition, Integer.parseInt(number));
				}
			}
		}
	}
	
	
	// Cardinalidades??
	private static void loadOutputStructureInSubstate(UID uid, String idState,
			String idSubstate, String nameStructure, Element elemStructure) {
		Node n = elemStructure.getFirstChild();
		while (n != null) {
			if (n.getNodeType() == Node.ELEMENT_NODE) {
				Element elem = (Element) n;
				if (elem.getTagName().equals("output-data-item")) {
					OutputDataItem dataItem = loadOutputDataItem(elem);
					uid.insertOutputDataItemInStructureInSubstate(idState,
							idSubstate, nameStructure, dataItem);
					// Loading the transitions
					List outputDataItemTransitionElements = _getChildElements(
							elem, "transition");
					for (int i = 0; i < outputDataItemTransitionElements.size(); i++) {
						Element dataItemTransitionElement = (Element) outputDataItemTransitionElements
								.get(i);
						String idTransition = _getRequiredAttribute(
								dataItemTransitionElement, "id");
						String target = _getRequiredAttribute(
								dataItemTransitionElement, "target");
						String targetType = _getImpliedAttribute(
								dataItemTransitionElement, "target-type",
								"state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputDataItemInStructureInSubstateToState(
							idState, idSubstate, nameStructure,dataItem.getName(), target,idTransition);
						else
							uid.insertTransitionFromOutputDataItemInStructureInSubstateToAnotherUID(
							idState, idSubstate, nameStructure,dataItem.getName(), target,idTransition);
						String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition, option);
						String condition = _getImpliedAttribute(
								dataItemTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
					}
				}
				if (elem.getTagName().equals("output-data-item-set")) {
					
					
					//INLCUI DAQUI
					List outputDataItemElement = _getChildElements(elem,"output-data-item");
					Element outputDataItemElement1 = (Element)outputDataItemElement.get(0);
					//ATE AQUI
					

					OutputDataItem dataItemSet = loadOutputDataItem(outputDataItemElement1);
					String min = _getRequiredAttribute(elem, "min-cardinality");
					String max = _getRequiredAttribute(elem, "max-cardinality");
					dataItemSet.definesAsSet();
					dataItemSet.setCardinalityMin(Integer.parseInt(min));
					dataItemSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertOutputDataItemInStructureInSubstate(idState,
							idSubstate, nameStructure, dataItemSet);
					// Loading the transitions
					List outputDataItemSetTransitionElements = _getChildElements(
							elem, "transition-from-set");
					for (int i = 0; i < outputDataItemSetTransitionElements
							.size(); i++) {
						Element dataItemSetTransitionElement = (Element) outputDataItemSetTransitionElements
								.get(i);
						String number = _getRequiredAttribute(dataItemSetTransitionElement, "number");
						List dataItemSetTransitionElements = _getChildElements(
								dataItemSetTransitionElement, "transition");
						Element dataItemTransitionElement = (Element) dataItemSetTransitionElements.get(0);
						String idTransition = _getRequiredAttribute(
								dataItemTransitionElement, "id");
						String target = _getRequiredAttribute(
								dataItemTransitionElement, "target");
						String targetType = _getImpliedAttribute(
								dataItemTransitionElement, "target-type","state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputDataItemInStructureInSubstateToState(
							idState, idSubstate, nameStructure,dataItemSet.getName(), target,idTransition);
						else
							uid.insertTransitionFromOutputDataItemInStructureInSubstateToAnotherUID(
								idState, idSubstate, nameStructure,dataItemSet.getName(), target,idTransition);
						String option = _getImpliedAttribute(
								dataItemTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition, option);
						String condition = _getImpliedAttribute(
								dataItemTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(
								dataItemTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
						uid.setNumberOfSelectedElements(idTransition, Integer.parseInt(number));
					}
				}
				if (elem.getTagName().equals("output-structure")) {
					String name = _getRequiredAttribute(elem, "name");
					OutputStructure structure = new OutputStructure(name);
					uid.insertOutputStructureInStructureInSubstate(idState,
							idSubstate, nameStructure, structure);
					List outputStructureElements = _getChildElements(elem,"output-structure-elements");
					Element outputStructureElement = (Element) outputStructureElements.get(0);
					loadOutputStructure(uid, idState, name,outputStructureElement);
					// Loading the transitions
					List outputStructureTransitionElements = _getChildElements(elem, "transition");
					for (int i = 0; i < outputStructureTransitionElements.size(); i++) {
						Element structureTransitionElement = (Element) outputStructureTransitionElements
								.get(i);
						String idTransition = _getRequiredAttribute(structureTransitionElement, "id");
						String target = _getRequiredAttribute(structureTransitionElement, "target");
						String targetType = _getImpliedAttribute(structureTransitionElement, "target-type","state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputStructureInStructureInSubstateToState(
								idState, idSubstate, nameStructure,name, target, idTransition);
						else
							uid.insertTransitionFromOutputStructureInStructureInSubstateToAnotherUID(
								idState, idSubstate, nameStructure,name, target, idTransition);
						String option = _getImpliedAttribute(structureTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition, option);
						String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return","false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
					}
				}
				if (elem.getTagName().equals("output-structure-set")) {
					String name = _getRequiredAttribute(elem, "name");
					OutputStructure structureSet = new OutputStructure(name);
					String min = _getRequiredAttribute(elem, "min-cardinality");
					String max = _getRequiredAttribute(elem, "max-cardinality");
					structureSet.setCardinalityMin(Integer.parseInt(min));
					structureSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertOutputStructureInStructureInSubstate(idState,
							idSubstate, nameStructure, structureSet);
					List outputStructureSetElements = _getChildElements(elem,"output-structure-elements");
					Element outputStructureSetElement = (Element) outputStructureSetElements.get(0);

					// Porque se voc est dentro de output?????

					loadInputStructure(uid, idState, name,outputStructureSetElement);
					// Loading the transitions
					List outputStructureSetTransitionElements = _getChildElements(
							elem, "transition-from-set");
					for (int i = 0; i < outputStructureSetTransitionElements.size(); i++) {
						Element structureSetTransitionElement = (Element) outputStructureSetTransitionElements.get(i);
						String number = _getRequiredAttribute(structureSetTransitionElement, "number");
						List structureSetTransitionElements = _getChildElements(structureSetTransitionElement, "transition");
						Element structureTransitionElement = (Element) structureSetTransitionElements.get(0);
						String idTransition = _getRequiredAttribute(structureTransitionElement, "id");
						String target = _getRequiredAttribute(structureTransitionElement, "target");
						String targetType = _getImpliedAttribute(structureTransitionElement, "target-type","state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputStructureInStructureInSubstateToState(
								idState, idSubstate, nameStructure,name, target, idTransition);
						else
							uid.insertTransitionFromOutputStructureInStructureInSubstateToAnotherUID(
								idState, idSubstate, nameStructure,name, target, idTransition);
						String option = _getImpliedAttribute(structureTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition, option);
						String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return","false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
						uid.setNumberOfSelectedElements(idTransition, Integer.parseInt(number));
					}
				}
			}
			n = n.getNextSibling();
		}
	}


	private static void loadDataEntryInSubstate(UID uid, String idState,
			String idSubstate, Element dataEntry) {
		Node n = dataEntry.getFirstChild();
		while (n == null || n.getNodeType() != Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if (n != null && n.getNodeType() == Node.ELEMENT_NODE) {
			Element elem = (Element) n;
			if (elem.getTagName().equals("optional-data-item")) {
				List optionalDataItemElements = _getChildElements(elem,"data-item");
				Element optionalDataItemElement = (Element) optionalDataItemElements.get(0);
				InputOptionalDataItem dataItem = loadInputOptionalDataItem(optionalDataItemElement);

				uid.insertInputOptionalDataItemInSubstate(idState, idSubstate,dataItem);
			}
			if (elem.getTagName().equals("data-item")) {
				InputDataItem dataItem = loadInputDataItem(elem);
				uid.insertInputDataItemInSubstate(idState, idSubstate,dataItem);
			}
			if (elem.getTagName().equals("OR-data-entry")) {

				List dataItens = _getChildElements(elem, "data-item");
				Element dataItem1 = (Element) dataItens.get(0);
				Element dataItem2 = (Element) dataItens.get(1);

				String nameORDataEntry1 = _getRequiredAttribute(dataItem1,"name");
				String domainORDataEntry1 = _getImpliedAttribute(dataItem1,"domain", null);
				String nameORDataEntry2 = _getRequiredAttribute(dataItem2,"name");
				String domainORDataEntry2 = _getImpliedAttribute(dataItem2,"domain", null);
				InputORDataEntry oRdataEntry;
				if (domainORDataEntry1 == "")
					oRdataEntry = new InputORDataEntry(nameORDataEntry1,nameORDataEntry2);
				else
					oRdataEntry = new InputORDataEntry(nameORDataEntry1,domainORDataEntry1, 
							nameORDataEntry2,domainORDataEntry2);
				uid.insertInputORDataItemInSubstate(idState, idSubstate,oRdataEntry);
			}
			if (elem.getTagName().equals("XOR-data-entry")) {

				List dataItens = _getChildElements(elem, "data-item");
				Element dataItem1 = (Element) dataItens.get(0);
				Element dataItem2 = (Element) dataItens.get(1);

				Node n1 = elem.getFirstChild();
				String nameXORDataEntry1 = _getRequiredAttribute(dataItem1,"name");
				String domainXORDataEntry1 = _getImpliedAttribute(dataItem1,"domain", null);
				Node n2 = n1.getNextSibling();
				String nameXORDataEntry2 = _getRequiredAttribute(dataItem2,"name");
				String domainXORDataEntry2 = _getImpliedAttribute(dataItem2,"domain", null);
				InputXORDataEntry xORdataEntry;
				if (domainXORDataEntry1 == "")
					xORdataEntry = new InputXORDataEntry(nameXORDataEntry1,nameXORDataEntry2);
				else
					xORdataEntry = new InputXORDataEntry(nameXORDataEntry1,
							domainXORDataEntry1, nameXORDataEntry2,domainXORDataEntry2);
				uid.insertInputXORDataItemInSubstate(idState, idSubstate,xORdataEntry);
			}
			if (elem.getTagName().equals("enumerated-data")) {
				String nameEnumeratedData = _getRequiredAttribute(elem, "name");
				InputEnumeratedData dataItem = new InputEnumeratedData(nameEnumeratedData);
				uid.insertInputEnumeratedDataInSubstate(idState, idSubstate,dataItem);
				String minEnumeratedData = _getImpliedAttribute(elem,"min-selection", null);
				String maxEnumeratedData = _getImpliedAttribute(elem,"max-selection", null);
				if (minEnumeratedData != "" && maxEnumeratedData != "") {
					int min = Integer.parseInt(minEnumeratedData);
					int max = Integer.parseInt(maxEnumeratedData);
					uid.insertSelectionInInputEnumeratedDataInSubstate(idState,
							idSubstate, nameEnumeratedData, min, max);
				}
				List enumeratedOptionElements = _getChildElements(elem,"enumerated-option");
				for (int i = 0; i < enumeratedOptionElements.size(); i++) {
					Element enumeratedOption = (Element) enumeratedOptionElements.get(i);
					String option = _getRequiredAttribute(enumeratedOption,"name");
					uid.insertOptionInInputEnumeratedDataInSubstate(idState,
							idSubstate, nameEnumeratedData, option);
				}
			}
			if (elem.getTagName().equals("data-item-set")) {
				List dataItemSetElements = _getChildElements(elem, "data-item");
				Element dataItemElement = (Element) dataItemSetElements.get(0);
				InputDataItem dataItem = loadInputDataItem(dataItemElement);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				dataItem.definesAsSet();
				dataItem.setCardinalityMin(min);
				dataItem.setCardinalityMax(max);
				uid.insertInputDataItemInSubstate(idState, idSubstate,dataItem);
			}
			if (elem.getTagName().equals("structure")) {
				String nameStructure = _getRequiredAttribute(elem, "name");
				InputStructure structure = new InputStructure(nameStructure);
				uid.insertInputStructureInSubstate(idState, idSubstate,structure);
				loadInputStructureInSubstate(uid, idState, idSubstate,nameStructure, elem);
			}
			if (elem.getTagName().equals("optional-structure")) {
				List optionalStructureElements = _getChildElements(elem,"structure");
				Element optionalStructureElement = (Element) optionalStructureElements.get(0);
				String nameStructure = _getRequiredAttribute(optionalStructureElement, "name");
				InputStructure structure = new InputStructure(nameStructure);
				structure.setOptional(true);
				uid.insertInputStructureInSubstate(idState, idSubstate,structure);
				loadInputStructureInSubstate(uid, idState, idSubstate,nameStructure, elem);
			}
			if (elem.getTagName().equals("structure-set")) {
				List structureSetElements = _getChildElements(elem, "structure");
				Element structureElement = (Element) structureSetElements.get(0);
				String nameStructureSet = _getRequiredAttribute(structureElement, "name");
				String optional = _getRequiredAttribute(structureElement,"optional");
				InputStructure structureSet = new InputStructure(nameStructureSet);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				structureSet.definesAsSet();
				structureSet.setCardinalityMin(min);
				structureSet.setCardinalityMax(max);
				if (optional.equals("true"))
					structureSet.setOptional(true);
				uid.insertInputStructureInSubstate(idState, idSubstate,structureSet);
				loadInputStructureInSubstate(uid, idState, idSubstate,nameStructureSet, structureElement);
			}
		}

	}
	
	private static void loadInputStructureInSubstate(UID uid, String idState,
			String idSubstate, String nameStructure, Element elemStructure) {
		Node n = elemStructure.getFirstChild();
		while (n != null) {
			if (n.getNodeType() == Node.ELEMENT_NODE) {
				Element elem = (Element) n;
				if (elem.getTagName().equals("data-item")) {
					InputDataItem dataItem = loadInputDataItem(elem);
					uid.insertInputDataItemInStructureInSubstate(idState,
							idSubstate, nameStructure, dataItem);
				}
				if (elem.getTagName().equals("structure")) {
					String name = _getRequiredAttribute(elem, "name");
					InputStructure structure = new InputStructure(name);
					uid.insertInputStructureInStructureInSubstate(idState,
							idSubstate, nameStructure, structure);
					loadInputStructureInSubstate(uid, idState, idSubstate,name, elem);
				}
				if (elem.getTagName().equals("data-item-set")) {
					List dataItemSetElements = _getChildElements(elem,"data-item");
					Element dataItemElement = (Element) dataItemSetElements.get(0);
					InputDataItem dataItemSet = loadInputDataItem(dataItemElement);
					String min = _getRequiredAttribute(elem, "min-cardinality");
					String max = _getRequiredAttribute(elem, "max-cardinality");
					dataItemSet.setCardinalityMin(Integer.parseInt(min));
					dataItemSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertInputDataItemInStructureInSubstate(idState,
							idSubstate, nameStructure, dataItemSet);
				}
				if (elem.getTagName().equals("structure-set")) {
					List structureSetElements = _getChildElements(elem,"structure");
					Element structureElement = (Element) structureSetElements.get(0);
					String name = _getRequiredAttribute(structureElement,"name");
					InputStructure structureSet = new InputStructure(name);
					String min = _getRequiredAttribute(elem, "min-cardinality");
					String max = _getRequiredAttribute(elem, "max-cardinality");
					structureSet.setCardinalityMin(Integer.parseInt(min));
					structureSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertInputStructureInStructureInSubstate(idState,
							idSubstate, nameStructure, structureSet);
					loadInputStructureInSubstate(uid, idState, idSubstate,
							name, structureElement);
				}
			}
			n = n.getNextSibling();
		}
	}


	private static InputOptionalDataItem loadInputOptionalDataItem(Element elem) {
		String nameDataItem = _getRequiredAttribute(elem, "name");
		String domainDataItem = _getImpliedAttribute(elem, "domain", null);
		InputOptionalDataItem dataItem;
		if (domainDataItem == "")
			dataItem = new InputOptionalDataItem(nameDataItem);
		else
			dataItem = new InputOptionalDataItem(nameDataItem, domainDataItem);
		return dataItem;
	}


	private static void loadTextInSubstate(UID uid, String idState,
			String idSubstate, Element elem) {
		String id = _getRequiredAttribute(elem, "id");
		String data = _getRequiredAttribute(elem, "data");
		String optional = _getRequiredAttribute(elem, "optional");
		uid.insertTextInSubstate(idState, idSubstate, id, data);
		if (optional.equals("true")) {
			uid.defineTextInSubstateAsOptional(idState, idSubstate, id);
		}
	}

	
	
	private static void loadText(UID uid, String idState, Element elem) {
		String id = _getRequiredAttribute(elem, "id");
		String data = _getRequiredAttribute(elem, "data");
		String optional = _getRequiredAttribute(elem, "optional");
		uid.insertText(idState, id, data);
		if (optional.equals("true")) {
			uid.defineTextAsOptional(idState, id);
		}
	}

	//VELHO private static void loadOptionalText (UID uid, String idState, Element elem){
	//VELHO String id = _getRequiredAttribute(elem,"id");
	//VELHO String data = _getRequiredAttribute(elem,"data");
	//VELHO uid.insertText(idState,id,data);
	//VELHO uid.defineTextAsOptional(idState,id);
	//VELHO }

	private static void loadTransition (UID uid, String idState, Element elem){
		String idTransition = _getRequiredAttribute(elem,"id");
		String target = _getRequiredAttribute(elem,"target");
		String targetType = _getImpliedAttribute(elem, "target-type", "state");
		if (targetType.equals("state"))
			uid.insertTransitionFromStateToState(idState,target,idTransition);
		else
			uid.insertTransitionFromStateToAnotherUID(idState,target,idTransition);
		String option = _getImpliedAttribute(elem, "option", null);
		if (option != "")
			uid.insertOptionInTransition(idTransition,option);
		String condition = _getImpliedAttribute(elem, "condition", null);
		if (condition != "")
			uid.insertConditionInTransition(idTransition,condition);
		String noReturn = _getImpliedAttribute(elem, "no-return", "false");
		if (noReturn.equals("true"))
			uid.setNoReturnToTransition(idTransition);
	}

	
	/**
	 * @param uid
	 * @param idState
	 * @param dataOutput
	 */
	private static void loadDataOutput (UID uid, String idState, Element dataOutput){
		Node n = dataOutput.getFirstChild();
		while (n == null || n.getNodeType()!=Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if (n != null && n.getNodeType()==Node.ELEMENT_NODE){
			Element elem = (Element)n;
			if (elem.getTagName().equals("output-data-item")){
				OutputDataItem dataItem = loadOutputDataItem(elem);
				uid.insertOutputDataItem(idState,dataItem);				
				//Loading the transitions from the data items
				List outputDataItemTransitionElements = _getChildElements(elem,"transition");
				for (int i=0; i<outputDataItemTransitionElements.size(); i++){
					Element dataItemTransitionElement = (Element)outputDataItemTransitionElements.get(i);
					String idTransition = _getRequiredAttribute(dataItemTransitionElement,"id");
					String target = _getRequiredAttribute(dataItemTransitionElement,"target");
					String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputDataItemToState(idState,dataItem.getName(),
								target,idTransition);
					else	
						uid.insertTransitionFromOutputDataItemToAnotherUID(idState,dataItem.getName(),
								target,idTransition);
					String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition,option);
					String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
				}	
			}
//VELHO			if (elem.getTagName().equals("output-optional-data-item")){
//VELHO				OutputDataItem optionalDataItem = loadOutputDataItem(elem);
//VELHO				optionalDataItem.definesAsOptional();
//VELHO				uid.insertOutputDataItem(idState,optionalDataItem);				
//VELHO				//Loading the transitions from the data items
//VELHO				List outputDataItemTransitionElements = _getChildElements(elem,"transition");
//VELHO				for (int i=0; i<outputDataItemTransitionElements.size(); i++){
//VELHO					Element dataItemTransitionElement = (Element)outputDataItemTransitionElements.get(i);
//VELHO					String idTransition = _getRequiredAttribute(dataItemTransitionElement,"id");
//VELHO					String target = _getRequiredAttribute(dataItemTransitionElement,"target");
//VELHO					String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
//VELHO					if (targetType.equals("state"))
//VELHO						uid.insertTransitionFromOutputDataItemToState(idState,optionalDataItem.getName(),
//VELHO								target,idTransition);
//VELHO					else	
//VELHO						uid.insertTransitionFromOutputDataItemToAnotherUID(idState,optionalDataItem.getName(),
//VELHO								target,idTransition);
//VELHO					String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
//VELHO					if (option != "")
//VELHO						uid.insertOptionInTransition(idTransition,option);
//VELHO					String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
//VELHO					if (condition != "")
//VELHO						uid.insertConditionInTransition(idTransition,condition);
//VELHO					String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
//VELHO					if (noReturn.equals("true"))
//VELHO						uid.setNoReturnToTransition(idTransition);
//VELHO				}	
//VELHO			}
			if (elem.getTagName().equals("output-data-item-set")){
				List dataItemSetElements = _getChildElements(elem,"output-data-item"); 	//NOVO
				Element elem1 = (Element) dataItemSetElements.get(0);					//NOVO
				OutputDataItem dataItem = loadOutputDataItem(elem1);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				dataItem.definesAsSet();
				dataItem.setCardinalityMin(min);
				dataItem.setCardinalityMax(max);
				uid.insertOutputDataItem(idState,dataItem);
				//Loading the transitions from the set of data items
				List outputDataItemSetTransitionElements = _getChildElements(elem,"transition-from-set");
				for (int i=0; i<outputDataItemSetTransitionElements.size(); i++){
					Element dataItemSetTransitionElement = (Element)outputDataItemSetTransitionElements.get(i);
					String number = _getRequiredAttribute(dataItemSetTransitionElement,"number");
					List dataItemSetTransitionElements = _getChildElements(dataItemSetTransitionElement,"transition");
					Element dataItemTransitionElement = (Element)dataItemSetTransitionElements.get(0);
					String idTransition = _getRequiredAttribute(dataItemTransitionElement,"id");
					String target = _getRequiredAttribute(dataItemTransitionElement,"target");
					String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputDataItemToState(idState,dataItem.getName(),target,idTransition);
					else
						uid.insertTransitionFromOutputDataItemToAnotherUID(idState,dataItem.getName(),target,idTransition);
					String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition,option);		
					String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
					uid.setNumberOfSelectedElements(idTransition,Integer.parseInt(number));
				}	
			}
			if (elem.getTagName().equals("output-structure")){
				String nameStructure = _getRequiredAttribute(elem,"name");
				String optional = _getRequiredAttribute(elem, "optional");		//NOVO
				OutputStructure structure = new OutputStructure (nameStructure);
				if (optional.equals("true"))			//NOVO
					structure.setOptional(true);		//NOVO
				uid.insertOutputStructure(idState,structure);
				List outputStructureElements = _getChildElements(elem,"output-structure-elements");
				Element outputStructureElement = (Element)outputStructureElements.get(0);
				loadOutputStructure (uid,idState,nameStructure,outputStructureElement);
				//Loading the transitions from the structure
				List outputStructureTransitionElements = _getChildElements(elem,"transition");
				for (int i=0; i<outputStructureTransitionElements.size(); i++){
					Element structureTransitionElement = (Element)outputStructureTransitionElements.get(i);
					String idTransition = _getRequiredAttribute(structureTransitionElement,"id");
					String target = _getRequiredAttribute(structureTransitionElement,"target");
					String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputStructureToState(idState,nameStructure,target,idTransition);
					else
						uid.insertTransitionFromOutputStructureToAnotherUID(idState,nameStructure,target,idTransition);
					String option = _getImpliedAttribute(structureTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition,option);
					String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
				}	
			}
		

/*antigo			if (elem.getTagName().equals("output-structure-set")) {
				String nameStructureSet = _getRequiredAttribute(elem, "name");
				String optional = _getRequiredAttribute(elem, "optional");
				OutputStructure structure = new OutputStructure(nameStructureSet);
				if (optional.equals("true"))
					structure.setOptional(true);
				int min = Integer.parseInt(_getRequiredAttribute(elem,
						"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,
						"max-cardinality"));
				structure.definesAsSet();
				structure.setCardinalityMin(min);
				structure.setCardinalityMax(max);
				uid.insertOutputStructure(idState, structure);
				List outputStructureSetElements = _getChildElements(elem,
						"output-structure-elements");
				Element outputStructureSetElement = (Element) outputStructureSetElements
						.get(0);
				loadOutputStructure(uid, idState, nameStructureSet,
						outputStructureSetElement);
*/
		
//VELHO			if (elem.getTagName().equals("output-optional-structure")){
/*				String nameStructure = _getRequiredAttribute(elem,"name");
				OutputStructure optionalStructure = new OutputStructure (nameStructure);
				optionalStructure.definesAsOptional();
				uid.insertOutputStructure(idState,optionalStructure);
				List outputStructureElements = _getChildElements(elem,"output-structure-elements");
				Element outputStructureElement = (Element)outputStructureElements.get(0);
				loadOutputStructure (uid,idState,nameStructure,outputStructureElement);
				//Loading the transitions from the structure
				List outputStructureTransitionElements = _getChildElements(elem,"transition");
				for (int i=0; i<outputStructureTransitionElements.size(); i++){
					Element structureTransitionElement = (Element)outputStructureTransitionElements.get(i);
					String idTransition = _getRequiredAttribute(structureTransitionElement,"id");
					String target = _getRequiredAttribute(structureTransitionElement,"target");
					String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputStructureToState(idState,nameStructure,target,idTransition);
					else
						uid.insertTransitionFromOutputStructureToAnotherUID(idState,nameStructure,target,idTransition);
					String option = _getImpliedAttribute(structureTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition,option);
					String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
				}	
			}
*/			
			if (elem.getTagName().equals("output-structure-set")){
				String nameStructureSet = _getRequiredAttribute(elem,"name");
				String optional = _getRequiredAttribute(elem, "optional");		//NOVO
				OutputStructure structure = new OutputStructure (nameStructureSet);
				if (optional.equals("true"))			//NOVO
					structure.setOptional(true);		//NOVO
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				structure.definesAsSet();
				structure.setCardinalityMin(min);
				structure.setCardinalityMax(max);
				uid.insertOutputStructure(idState,structure);
				List outputStructureSetElements = _getChildElements(elem,"output-structure-elements");
				Element outputStructureSetElement = (Element)outputStructureSetElements.get(0);
				loadOutputStructure (uid,idState,nameStructureSet,outputStructureSetElement);

				//Loading the transitions from the set of structures
				List outputStructureSetTransitionElements = _getChildElements(elem,"transition-from-set");
				for (int i=0; i<outputStructureSetTransitionElements.size(); i++){
					Element structureSetTransitionElement = (Element)outputStructureSetTransitionElements.get(i);
					String number = _getRequiredAttribute(structureSetTransitionElement,"number");
					List structureSetTransitionElements = _getChildElements(structureSetTransitionElement,"transition");
					Element structureTransitionElement = (Element)structureSetTransitionElements.get(0);
					String idTransition = _getRequiredAttribute(structureTransitionElement,"id");
					String target = _getRequiredAttribute(structureTransitionElement,"target");
					String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
					if (targetType.equals("state"))
						uid.insertTransitionFromOutputStructureToState(idState,nameStructureSet,target,idTransition);
					else
						uid.insertTransitionFromOutputStructureToAnotherUID(idState,nameStructureSet,target,idTransition);
					String option = _getImpliedAttribute(structureTransitionElement, "option", null);
					if (option != "")
						uid.insertOptionInTransition(idTransition,option);		
					String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
					if (condition != "")
						uid.insertConditionInTransition(idTransition,condition);
					String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
					if (noReturn.equals("true"))
						uid.setNoReturnToTransition(idTransition);
					uid.setNumberOfSelectedElements(idTransition,Integer.parseInt(number));
				}	
			}
		}
	}	
		
	private static OutputDataItem loadOutputDataItem (Element elem){
		String nameDataItem = _getRequiredAttribute(elem,"name");
		String domainDataItem = _getImpliedAttribute(elem, "domain", null);
		String optional = _getRequiredAttribute(elem, "optional");
		OutputDataItem dataItem;
		if (domainDataItem == "")
			dataItem = new OutputDataItem (nameDataItem);
		else
			dataItem = new OutputDataItem (nameDataItem, domainDataItem);
		if (optional.equals("true"))
			dataItem.setOptional(true);
		return dataItem;
	}
	
	
	// Filipe: Falta fazer!! Cardinalidades??
	private static void loadOutputStructure (UID uid, String idState, String nameStructure, Element elemStructure){
		Node n = elemStructure.getFirstChild();
		while (n != null){
			if (n.getNodeType()==Node.ELEMENT_NODE){
				Element elem = (Element)n;
				if (elem.getTagName().equals("output-data-item")){
					OutputDataItem dataItem = loadOutputDataItem (elem);
					uid.insertOutputDataItemInStructure(idState,nameStructure,dataItem);
					//Loading the transitions 
					List outputDataItemTransitionElements = _getChildElements(elem,"transition");
					for (int i=0; i<outputDataItemTransitionElements.size(); i++){
						Element dataItemTransitionElement = (Element)outputDataItemTransitionElements.get(i);
						String idTransition = _getRequiredAttribute(dataItemTransitionElement,"id");
						String target = _getRequiredAttribute(dataItemTransitionElement,"target");
						String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputDataItemInStructureToState(idState,dataItem.getName(),
									nameStructure,target,idTransition);
						else
							uid.insertTransitionFromOutputDataItemInStructureToAnotherUID(idState,dataItem.getName(),
									nameStructure,target,idTransition);
						String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition,option);
						String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
					}	
				}
				if (elem.getTagName().equals("output-data-item-set")){
					
					//INLCUI DAQUI
					List outputDataItemElement = _getChildElements(elem,"output-data-item");
					Element outputDataItemElement1 = (Element)outputDataItemElement.get(0);
					//ATE AQUI
					
					OutputDataItem dataItemSet = loadOutputDataItem(outputDataItemElement1);		
					//OutputDataItem dataItemSet = loadOutputDataItem(elem);		
					String min = _getRequiredAttribute(elem,"min-cardinality");
					String max = _getRequiredAttribute(elem,"max-cardinality");
					dataItemSet.definesAsSet();
					dataItemSet.setCardinalityMin(Integer.parseInt(min));
					dataItemSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertOutputDataItemInStructure(idState,nameStructure,dataItemSet);					
					//Loading the transitions
					List outputDataItemSetTransitionElements = _getChildElements(elem,"transition-from-set");
					for (int i=0; i<outputDataItemSetTransitionElements.size(); i++){
						Element dataItemSetTransitionElement = (Element)outputDataItemSetTransitionElements.get(i);
						String number = _getRequiredAttribute(dataItemSetTransitionElement,"number");
						List dataItemSetTransitionElements = _getChildElements(dataItemSetTransitionElement,"transition");
						Element dataItemTransitionElement = (Element)dataItemSetTransitionElements.get(0);
						String idTransition = _getRequiredAttribute(dataItemTransitionElement,"id");
						String target = _getRequiredAttribute(dataItemTransitionElement,"target");
						String targetType = _getImpliedAttribute(dataItemTransitionElement, "target-type", "state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputDataItemInStructureToState(idState,dataItemSet.getName(),
									nameStructure,target,idTransition);
						else
							uid.insertTransitionFromOutputDataItemInStructureToAnotherUID(idState,dataItemSet.getName(),
									nameStructure,target,idTransition);
						String option = _getImpliedAttribute(dataItemTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition,option);		
						String condition = _getImpliedAttribute(dataItemTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(dataItemTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
						uid.setNumberOfSelectedElements(idTransition,Integer.parseInt(number));
					}	
				}
				if (elem.getTagName().equals("output-structure")){
					String name = _getRequiredAttribute(elem,"name");
					OutputStructure structure = new OutputStructure (name);			
					uid.insertOutputStructureInStructure(idState,nameStructure,structure);
					List outputStructureElements = _getChildElements(elem,"output-structure-elements");
					Element outputStructureElement = (Element)outputStructureElements.get(0);
					loadOutputStructure(uid,idState,name,outputStructureElement);
					//Loading the transitions
					List outputStructureTransitionElements = _getChildElements(elem,"transition");
					for (int i=0; i<outputStructureTransitionElements.size(); i++){
						Element structureTransitionElement = (Element)outputStructureTransitionElements.get(i);
						String idTransition = _getRequiredAttribute(structureTransitionElement,"id");
						String target = _getRequiredAttribute(structureTransitionElement,"target");
						String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputStructureInStructureToState(idState,name,nameStructure,
									target,idTransition);
						else
							uid.insertTransitionFromOutputStructureInStructureToAnotherUID(idState,name,nameStructure,
									target,idTransition);
						String option = _getImpliedAttribute(structureTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition,option);
						String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
					}	
				}
				if (elem.getTagName().equals("output-structure-set")){
					String name = _getRequiredAttribute(elem,"name");
					OutputStructure structureSet = new OutputStructure (name);
					String min = _getRequiredAttribute(elem,"min-cardinality");
					String max = _getRequiredAttribute(elem,"max-cardinality");
					structureSet.setCardinalityMin(Integer.parseInt(min));
					structureSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertOutputStructureInStructure(idState,nameStructure,structureSet);
					List outputStructureSetElements = _getChildElements(elem,"output-structure-elements");
					Element outputStructureSetElement = (Element)outputStructureSetElements.get(0);
					loadOutputStructure(uid,idState,name,outputStructureSetElement);
					//Loading the transitions
					List outputStructureSetTransitionElements = _getChildElements(elem,"transition-from-set");
					for (int i=0; i<outputStructureSetTransitionElements.size(); i++){
						Element structureSetTransitionElement = (Element)outputStructureSetTransitionElements.get(i);
						String number = _getRequiredAttribute(structureSetTransitionElement,"number");
						List structureSetTransitionElements = _getChildElements(structureSetTransitionElement,"transition");
						Element structureTransitionElement = (Element)structureSetTransitionElements.get(0);
						String idTransition = _getRequiredAttribute(structureTransitionElement,"id");
						String target = _getRequiredAttribute(structureTransitionElement,"target");
						String targetType = _getImpliedAttribute(structureTransitionElement, "target-type", "state");
						if (targetType.equals("state"))
							uid.insertTransitionFromOutputStructureInStructureToState(idState,name,nameStructure,
									target,idTransition);
						else
							uid.insertTransitionFromOutputStructureInStructureToAnotherUID(idState,name,nameStructure,
									target,idTransition);
						String option = _getImpliedAttribute(structureTransitionElement, "option", null);
						if (option != "")
							uid.insertOptionInTransition(idTransition,option);		
						String condition = _getImpliedAttribute(structureTransitionElement, "condition", null);
						if (condition != "")
							uid.insertConditionInTransition(idTransition,condition);
						String noReturn = _getImpliedAttribute(structureTransitionElement, "no-return", "false");
						if (noReturn.equals("true"))
							uid.setNoReturnToTransition(idTransition);
						uid.setNumberOfSelectedElements(idTransition,Integer.parseInt(number));
					}	
				}
			}
			n = n.getNextSibling();
		}
	}

	
	private static void loadDataEntry (UID uid, String idState, Element dataEntry){
		Node n = dataEntry.getFirstChild();
		while (n == null || n.getNodeType()!=Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if (n != null && n.getNodeType()==Node.ELEMENT_NODE){
			Element elem = (Element)n;
			if (elem.getTagName().equals("optional-data-item")){
				List optionalDataItemElements = _getChildElements(elem,"data-item");
				Element optionalDataItemElement = (Element)optionalDataItemElements.get(0);
				InputOptionalDataItem dataItem = loadInputOptionalDataItem(optionalDataItemElement);  //NOVO		
//VELHO				InputDataItem dataItem = loadInputDataItem (optionalDataItemElement);
//VELHO				dataItem.definesAsOptional();
				uid.insertInputOptionalDataItem(idState,dataItem);
			}
			if (elem.getTagName().equals("data-item")){
				InputDataItem dataItem = loadInputDataItem (elem);
				uid.insertInputDataItem(idState,dataItem);
			}
			if (elem.getTagName().equals("OR-data-entry")){
				//MUDOU
				List dataItens = _getChildElements(elem, "data-item");		//NOVO
				Element dataItem1 = (Element) dataItens.get(0);				//NOVO
				Element dataItem2 = (Element) dataItens.get(1);				//NOVO
				String nameORDataEntry1 = _getRequiredAttribute(dataItem1, "name");				//NOVO
				String domainORDataEntry1 = _getImpliedAttribute(dataItem1,	"domain", null);	//NOVO
				String nameORDataEntry2 = _getRequiredAttribute(dataItem2,	"name");			//NOVO
				String domainORDataEntry2 = _getImpliedAttribute(dataItem2, "domain", null);	//NOVO
				InputORDataEntry oRdataEntry;
				if (domainORDataEntry1 == "")
					oRdataEntry = new InputORDataEntry (nameORDataEntry1,nameORDataEntry2);
				else
					oRdataEntry = new InputORDataEntry (nameORDataEntry1,domainORDataEntry1,
							nameORDataEntry2,domainORDataEntry2);
				uid.insertInputORDataItem(idState,oRdataEntry);
			}
			if (elem.getTagName().equals("XOR-data-entry")){
				List dataItens = _getChildElements(elem, "data-item");	//NOVO
				Element dataItem1 = (Element) dataItens.get(0);		//NOVO
				Element dataItem2 = (Element) dataItens.get(1);		//NOVO
				// Node n1 = elem.getFirstChild();
				String nameXORDataEntry1 = _getRequiredAttribute(dataItem1,"name");
				String domainXORDataEntry1 = _getImpliedAttribute(dataItem1,"domain", null);
				// Node n2 = n1.getNextSibling();
				String nameXORDataEntry2 = _getRequiredAttribute(dataItem2,"name");
				String domainXORDataEntry2 = _getImpliedAttribute(dataItem2,"domain", null);
				InputXORDataEntry xORdataEntry;
				if (domainXORDataEntry1 == "")
					xORdataEntry = new InputXORDataEntry (nameXORDataEntry1,nameXORDataEntry2);
				else
					xORdataEntry = new InputXORDataEntry (nameXORDataEntry1,domainXORDataEntry1,
							nameXORDataEntry2,domainXORDataEntry2);
				uid.insertInputXORDataItem(idState,xORdataEntry);
			}
			if (elem.getTagName().equals("enumerated-data")){
				String nameEnumeratedData = _getRequiredAttribute(elem, "name");
				InputEnumeratedData dataItem = new InputEnumeratedData(nameEnumeratedData);
				uid.insertInputEnumeratedData(idState,dataItem);
				String minEnumeratedData = _getImpliedAttribute(elem, "min-selection", null);					
				String maxEnumeratedData = _getImpliedAttribute(elem, "max-selection", null);
				if (minEnumeratedData!="" && maxEnumeratedData!=""){
					int min = Integer.parseInt(minEnumeratedData);
					int max = Integer.parseInt(maxEnumeratedData);
					uid.insertSelectionInInputEnumeratedData(idState,nameEnumeratedData,min,max);
				}
				List enumeratedOptionElements = _getChildElements(elem,"enumerated-option");
				for (int i=0; i<enumeratedOptionElements.size(); i++){
					Element enumeratedOption = (Element)enumeratedOptionElements.get(i);
					String option = _getRequiredAttribute(enumeratedOption,"name");
					uid.insertOptionInInputEnumeratedData(idState,nameEnumeratedData,option);
				}	
			}
			if (elem.getTagName().equals("data-item-set")){
				List dataItemSetElements = _getChildElements(elem,"data-item");
				Element dataItemElement = (Element)dataItemSetElements.get(0);
				InputDataItem dataItem = loadInputDataItem(dataItemElement);		
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				dataItem.definesAsSet();
				dataItem.setCardinalityMin(min);
				dataItem.setCardinalityMax(max);
				uid.insertInputDataItem(idState,dataItem);
			}
			if (elem.getTagName().equals("structure")){
				String nameStructure = _getRequiredAttribute(elem,"name");
				InputStructure structure = new InputStructure (nameStructure);
				uid.insertInputStructure(idState,structure);
				loadInputStructure (uid,idState,nameStructure,elem);
			}
			if (elem.getTagName().equals("optional-structure")){
				List optionalStructureElements = _getChildElements(elem,"structure");
				Element optionalStructureElement = (Element)optionalStructureElements.get(0);
				String nameStructure = _getRequiredAttribute(optionalStructureElement,"name");
				InputStructure structure = new InputStructure (nameStructure);
//VELHO				structure.definesAsOptional();
				structure.setOptional(true);		//NOVO
				uid.insertInputStructure(idState,structure);
				loadInputStructure (uid,idState,nameStructure,elem);
			}
			if (elem.getTagName().equals("structure-set")){
				List structureSetElements = _getChildElements(elem,"structure");
				Element structureElement = (Element)structureSetElements.get(0);
				String nameStructureSet = _getRequiredAttribute(structureElement,"name");
				String optional = _getRequiredAttribute(structureElement,"optional");
				InputStructure structureSet = new InputStructure (nameStructureSet);
				int min = Integer.parseInt(_getRequiredAttribute(elem,"min-cardinality"));
				int max = Integer.parseInt(_getRequiredAttribute(elem,"max-cardinality"));
				structureSet.definesAsSet();
				structureSet.setCardinalityMin(min);
				structureSet.setCardinalityMax(max);
				if (optional.equals("true"))
					structureSet.setOptional(true);
				uid.insertInputStructure(idState,structureSet);
				loadInputStructure (uid,idState,nameStructureSet,structureElement);					
			}
		}
	}

	private static InputDataItem loadInputDataItem (Element elem){
		String nameDataItem = _getRequiredAttribute(elem,"name");
		String domainDataItem = _getImpliedAttribute(elem, "domain", null);
		InputDataItem dataItem;
		if (domainDataItem == "")
			dataItem = new InputDataItem (nameDataItem);
		else
			dataItem = new InputDataItem (nameDataItem, domainDataItem);
		return dataItem;
	}
	
	
	private static void loadInputStructure (UID uid, String idState, String nameStructure, Element elemStructure){
		Node n = elemStructure.getFirstChild();
		while (n != null){
			if (n.getNodeType()==Node.ELEMENT_NODE){
				Element elem = (Element)n;
				if (elem.getTagName().equals("data-item")){
					InputDataItem dataItem = loadInputDataItem (elem);
					uid.insertInputDataItemInStructure(idState,nameStructure,dataItem);
				}
				if (elem.getTagName().equals("structure")){
					String name = _getRequiredAttribute(elem,"name");
					InputStructure structure = new InputStructure (name);			
					uid.insertInputStructureInStructure(idState,nameStructure,structure);
					loadInputStructure(uid,idState,name,elem);
				}
				if (elem.getTagName().equals("data-item-set")){
					List dataItemSetElements = _getChildElements(elem,"data-item");
					Element dataItemElement = (Element)dataItemSetElements.get(0);
					InputDataItem dataItemSet = loadInputDataItem(dataItemElement);		
					String min = _getRequiredAttribute(elem,"min-cardinality");
					String max = _getRequiredAttribute(elem,"max-cardinality");
					dataItemSet.setCardinalityMin(Integer.parseInt(min));
					dataItemSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertInputDataItemInStructure(idState,nameStructure,dataItemSet);
				}
				if (elem.getTagName().equals("structure-set")){
					List structureSetElements = _getChildElements(elem,"structure");
					Element structureElement = (Element)structureSetElements.get(0);
					String name = _getRequiredAttribute(structureElement,"name");
					InputStructure structureSet = new InputStructure (name);
					String min = _getRequiredAttribute(elem,"min-cardinality");
					String max = _getRequiredAttribute(elem,"max-cardinality");
					structureSet.setCardinalityMin(Integer.parseInt(min));
					structureSet.setCardinalityMax(Integer.parseInt(max));
					uid.insertInputStructureInStructure(idState,nameStructure,structureSet);
					loadInputStructure(uid,idState,name,structureElement);
				}
			}
			n = n.getNextSibling();
		}	
	}

	private static List _getChildElements(Element element, String tagName) {
		List<Node> nodeList = new ArrayList<Node>();
		Node n = element.getFirstChild();
		while (n != null) {
			if (n.getNodeType() == Node.ELEMENT_NODE && tagName.equals(((Element)n).getTagName())) {
				nodeList.add(n);
			}
			n = n.getNextSibling();
		}
		return nodeList;
	}


	private static String _getRequiredAttribute(NamedNodeMap attrs, String attrName) {
		try {
			Node val = attrs.getNamedItem(attrName);
			if (val != null && val instanceof Attr) {
				String value = ((Attr)val).getValue();
				if (value != null && value.trim().length() > 0 && !value.equals("null")) {
					return value;
				}
			}
		} catch (Exception e) {}
		throw new UIDException("cannot obtain value for " + attrName);
	}


	private static String _getImpliedAttribute(Element elem, String attrName, String defaultValue) {
		try {
			String value = elem.getAttribute(attrName);
			if (value != null) {
				return "null".equals(value) ? null : value;
			}
		} catch (Exception ex) {}
		return defaultValue;
	}
	
	
	private static String _getRequiredAttribute(Element elem, String attrName) {
		try {
			String value = elem.getAttribute(attrName);
			if (value != null && value.trim().length() > 0 && !value.equals("null")) {
				return value;
			}
		} catch (Exception ex) {}
		throw new UIDException("cannot obtain value for " + attrName);
	}
}
