package br.ufsc.uid2uml.UMLDiagram;

import java.util.*;

import br.ufsc.uid2uml.DictionaryEntry;

public class Class {
	
	private String name;
	private Class superClass;
	private List<Class> subClasses;
	private List<Attribute> attributes;
	private List<Association> associations;
	
	public Class (String name){
		this.name = name;
		this.subClasses = new ArrayList<Class> ();
		this.attributes = new ArrayList<Attribute> ();
		this.associations = new ArrayList<Association> ();
	}

	public void setSuperClass (Class superClass){
		this.superClass = superClass;
	}

	public void setSubClass (Class subClass){
		this.subClasses.add(subClass);
	}

	public void setAssociation(Association association) {
		this.associations.add(association);
	}

	public void setAttribute(Attribute attribute) {
		this.attributes.add(attribute);
	}

	public String getName() {
		return name;
	}

	public Class getSuperClass (){
		return this.superClass;
	}
	
	public List<Class> getSubClasses (){
		return this.subClasses;
	}
	
	public List<Association> getAssociations() {
		return associations;
	}

	public List<Attribute> getAttributes() {
		return attributes;
	}
	
	public boolean containsAttribute (String attributeName){
		for (int i=0, size=this.attributes.size(); i<size; i++){
			Attribute attribute = this.attributes.get(i);
			if (attribute.getName().equals(attributeName))
				return true;
		}
		return false;
	}

	public boolean containsAssociationWith (String className){
		for (int i=0, size=this.associations.size(); i<size; i++){
			Association association = this.associations.get(i);
			if ((association.getSourceClass().getName().equals(this.getName()) &&
			     association.getTargetClass().getName().equals(className))  ||
			    (association.getSourceClass().getName().equals(className) &&
				 association.getTargetClass().getName().equals(this.getName())) )
				return true;
		}
		return false;
	}
	
	public boolean containsAssociationWithClassAndSameAssociationName (String className, String associationName){
		for (int i=0, size=this.associations.size(); i<size; i++){
			Association association = this.associations.get(i);
			if ((association.getSourceClass().getName().equals(this.getName()) &&
			     association.getTargetClass().getName().equals(className) &&
			     association.getName().equals(associationName))  ||
			    (association.getSourceClass().getName().equals(className) &&
				 association.getTargetClass().getName().equals(this.getName()) &&
				 association.getName().equals(associationName)) )
				return true;
		}
		return false;
	}
	
	
	public Association getAssociationWithClassAndSameAssociationName (String className, String associationName){
		for (int i=0, size=this.associations.size(); i<size; i++){
			Association association = this.associations.get(i);
			if ((association.getSourceClass().getName().equals(this.getName()) &&
			     association.getTargetClass().getName().equals(className) &&
			     association.getName().equals(associationName))  ||
			    (association.getSourceClass().getName().equals(className) &&
				 association.getTargetClass().getName().equals(this.getName()) &&
				 association.getName().equals(associationName)) )
				return association;
		}
		return null;
	}
	
	
	public DictionaryEntry[] getDictionaryEntries() {	
		List<DictionaryEntry> dictEntries = new ArrayList<DictionaryEntry>();
		dictEntries.add(new DictionaryEntry(this.name,null,"class"));
		for (int i=0, size=this.attributes.size(); i<size; i++){
			Attribute attribute = this.attributes.get(i);
			dictEntries.add(new DictionaryEntry(attribute.getName(), "class "+this.name, "attribute"));
		}
		return dictEntries.toArray(new DictionaryEntry[dictEntries.size()]);
	}		

	

}
