package br.ufsc.uid2uml.UMLDiagram;

import java.util.*;

import br.ufsc.uid2uml.DictionaryEntry;


public class ClassDiagram {
	
	private String name;
	private List<Class> classes;
	private List<Association> associations;
	private List<Attribute> attributes;
	
	public ClassDiagram (String name){
		this.name = name;
		this.classes = new ArrayList<Class> ();
		this.associations = new ArrayList<Association> ();
		this.attributes = new ArrayList<Attribute> ();
	}
	
	public void insertClass (String className){
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className)){
				throw new ClassDiagramException (ClassDiagramException.EXISTING_CLASS, 
				"There is already a class with this name");
			}	
		}
		Class cl = new Class(className);
		this.classes.add(cl);
	}
	
	public Association insertAssociationBetweenClasses(String associationName, String sourceClassName, String targetClassName) {
		//precisava verificar se nao existe associacao entre as duas classes com o mesmo nome.
		Class sourceClass = null;
		Class targetClass = null;
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(sourceClassName))
				sourceClass = cl;
			if (cl.getName().equals(targetClassName))
				targetClass = cl;
		}
		if (sourceClass != null && targetClass != null)	{
			// Por enquanto s est verificando se existe uma associao qualquer entre as duas classes.
			// Deveria verificar se no existe associao com o mesmo nome, pois poderiam existir associaes 
			// com nomes diferentes entre as mesmas classes.
			if (!sourceClass.containsAssociationWithClassAndSameAssociationName(targetClass.getName(), associationName) &&
				!targetClass.containsAssociationWithClassAndSameAssociationName(sourceClass.getName(), associationName)) {
				Association a = new Association(sourceClass, targetClass, 1, 1, 1, 1);
				a.setName(associationName);
				sourceClass.setAssociation(a);
				targetClass.setAssociation(a);
				this.associations.add(a);
				return a;
			}
			else
				return null;
		}
		else
			throw new ClassDiagramException (ClassDiagramException.EXISTING_CLASS, 
			"There is no class with the name");
		
	}


	public void insertAttributeInClass(String attributeName, String className) {
		Class aClass = null;
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className))
				aClass = cl;
		}
		if (aClass != null)	{
			List<Attribute> attributes = aClass.getAttributes();
			for (int i = 0, size = attributes.size(); i < size; i++){
				Attribute a = attributes.get(i);
		//		if (a.getName().equals(className))  COMPAREI ERRADO, ERA PARA COMPARAR COM O NOME DO ATRIBUTO, NAO?
				if (a.getName().equals(attributeName))
					throw new ClassDiagramException (ClassDiagramException.EXISTING_ATTRIBUTE, 
					"There is already an attribute with this name");
			}	
			Attribute attribute = new Attribute(attributeName);	
			aClass.setAttribute(attribute);
			this.attributes.add(attribute);
		}
		else
			throw new ClassDiagramException (ClassDiagramException.EXISTING_CLASS, 
			"There is no class with the name");
	}


	public String getName() {
		return name;
	}

	public List<Class> getClasses() {
		return classes;
	}

	public List<Association> getAssociations() {
		return associations;
	}

	public List<Attribute> getAttributes() {
		return attributes;
	}

	public List<Attribute> getAttributesOfClass(String className) {
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className))
				return cl.getAttributes();
		}
		throw new ClassDiagramException (ClassDiagramException.EXISTING_CLASS, 
			"There is no class with the name");
	}	

	public List<Association> getAssociationsOfClass(String className) {
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className))
				return cl.getAssociations();
		}
		throw new ClassDiagramException (ClassDiagramException.EXISTING_CLASS, 
			"There is no class with the name");
	}	

	public boolean containsAttribute (String attributeName){
		for (int i = 0, size = this.attributes.size(); i < size; i++){
			Attribute attribute = this.attributes.get(i);
			if (attribute.getName().equals(attributeName))
				return true;
		}	
		return false;
	}
	
	public void removeClass (String className){
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className)){
				this.classes.remove(i);
				return;
			}	
		}		
	}
	
	public Class getClass (String className){
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			if (cl.getName().equals(className)){
				return cl;
			}	
		}		
		return null;
	}

	public DictionaryEntry[] getDictionaryEntries() {
		List<DictionaryEntry> dictEntries = new ArrayList<DictionaryEntry>();
		for (int i = 0, size = this.classes.size(); i < size; i++){
			Class cl = this.classes.get(i);
			dictEntries.addAll (Arrays.asList(cl.getDictionaryEntries()));
		}
		for (int i = 0, size = this.associations.size(); i < size; i++){
			Association as = this.associations.get(i);
			dictEntries.add(new DictionaryEntry(as.getSourceClass().getName()+" "+as.getName()+" "+as.getTargetClass().getName(),
					                            null,"association"));
		}
		return dictEntries.toArray(new DictionaryEntry[dictEntries.size()]);
	}	
	
}
